        .MODEL tiny
        .DATA
;---------------------------------------------------;
; data
;---------------------------------------------------;
hlp     db      'COL 1.2 - Copyright (C) M. R. Le Voi Systems Consultants',cr,lf,lf
        db      'Syntax: COL [e|v] [i] [n] [h] [N1] [N2] [/help]'
        db      cr,lf,lf
        db      'Where:-',cr,lf
        db      '   e  - set 43 line EGA mode/50 line VGA mode',cr,lf
        db      '   v  - set 28 line EGA mode on VGA monitor',cr,lf
        db      '   i  - enable high intensity background',cr,lf
        db      '   n  - disable high background intensity',cr,lf
        db      '   h  - high intensity colour',cr,lf
        db      '   N1 - foreground colour number',cr,lf
        db      '   N2 - background colour number',cr,lf,lf
        db      'Colour numbers are as follows:-',cr,lf
        db      '   0=black  1=blue  2=green  3=cyan',cr,lf
        db      '   4=red    5=pink  6=brown  7=white',cr,lf,lf
        db      'If no numbers are specified, default is cyan on black',cr,lf,lf
        db      'Examples:',cr,lf
        db      '   COL e 71    - Set EGA/VGA mode and color to white on blue ',cr,lf
        db      '   COL i h7h1  - Set high intensity white on hi blue',cr,lf
        db      '$'
fore    db      3               ; default foreground
back    db      0               ; default background
ega     dw      0               ; ega/cga flag word
egaflag equ     80h             ; ega flag
egasize equ     40h             ; 43 line flag
vgaflag equ     20h             ; 28 line VGA flag
cr      equ     0dh
lf      equ     0ah
        .CODE
        .STARTUP
;---------------------------------------------------;
; COL.COM - a command to set colour and screen size
; Copyright   M.R.  Le Voi Systems Consultants 1988, 1993
;---------------------------------------------------;
        mov     si,80h          ; point to command tail
        xor     cx,cx
        mov     cl,byte ptr [si] ; use si as pointer
        cmp     cx,0            ; no tail?
        je      noparm          ; take defaults
        mov     ah,0
        mov     di,offset fore  ; point to foreground byte
nextchr:
        inc     si              ; point to next char in tail
        mov     al,byte ptr [si]
        cmp     al,30h          ; is it 0 or above
        jb      tryflag
        cmp     al,37h          ; is it 7 or below
        ja      tryflag
        sub     al,30h          ; convert to binary
        or      al,bl           ; set high intensity bit
        stosb                   ; save char
        xor     bl,bl           ; zero out bl for next time
        inc     ah
        cmp     ah,2            ; done background yet?
        je      noparm          ; if equal, set color
        jmp     short more      ; else, check next char
tryflag:
        or      al,20h          ; convert input to lower case
        cmp     al,'/'          ; help wanted?
        jne     tryvga
        mov     dx,offset hlp   ; help message
        mov     ah,9            ; fn 9  - output string
        int     21h             ; DOS call
        jmp     endup
tryvga:
        cmp     al,'v'          ; 28 line mode wanted?
        jne     tryhi
;       or      ega,vgaflag     ; turn on vga 28 flag
;       jmp     short more
        jmp     vga28
tryhi:
        cmp     al,'i'          ; Enable high intensity background?
        jne     trylow
        push    ax
        push    bx
        mov     ah,10h          ; BIOS function 10h
        mov     al,3
        mov     bl,0
        int     10h             ; BIOS call
        pop     bx
        pop     ax
trylow:
        cmp     al,'n'          ; Disable high intensity background?
        jne     tryega
        push    ax
        push    bx
        mov     ah,10h          ; BIOS function 10h
        mov     al,3
        mov     bl,1
        int     10h             ; BIOS call
        pop     bx
        pop     ax
tryega:
        cmp     al,'e'          ; 43 line mode wanted?
        jne     tryhigh
        or      ega,egaflag     ; turn on ega flag
        jmp     short more
tryhigh:
        xor     bl,bl           ; zero out bl for next time
        cmp     al,'h'          ; high intensity wanted?
        jne     more
        mov     bl,08h          ; set high intensity bit
more:
        loop    nextchr
noparm:
        mov     ax,1130h        ; fn 11 = examine screen size
        int     10h             ; BIOS call
        cmp     dl,42           ; are we in 43 line mode or above?
        jb      testline        ; no, carry on
        or      ega,egasize     ; turn on 43 line flag
testline:
        test    ega,egaflag     ; 43 line wanted?
        jnz     line43          ; yes, so set 43 line mode
        test    ega,egasize     ; are we in 43 line mode?
        jz      csize           ; no, so carry on
        mov     ah,00h          ; fn 0 = set video mode
        mov     al,03h          ; 3 = 25x80 colour text
        int     10h             ; BIOS call
        jmp     short csize
line43:
        test    ega,egasize     ; are we in 43 line mode?
        jnz     csize           ; yes, so carry on
        mov     ax,1112h        ; fn 11 = set 43 line mode
        mov     bl,00
        int     10h             ; BIOS call
csize:
        mov     ah,01           ; fn 1 = set cursor size
        mov     cx,0607h        ; start csr = 06, end csr = 07
        test    ega,egaflag     ; 43 line wanted?
        jz      cursor          ; no, use small cursor
        mov     cx,0007h        ; start csr = 0, end csr = 07
cursor:
        int     10h             ; BIOS call
        mov     ah,2            ; function 2 = set cursor position
        mov     bh,0            ; page 0
        mov     dl,0            ; row 0
        mov     dh,0            ; col 0
        int     10h             ; BIOS call
        mov     al,back         ; shift background colour
        mov     cx,4            ; to left 4 bits
        shl     ax,cl           ; of al
        add     al,fore         ; add foreground attribute
        mov     bh,al           ; put attribute byte in bh
        mov     ah,6            ; function 6 = scroll up
        mov     al,0            ; 0 means clear screen
        mov     cx,0            ; start row/col 0,0
        mov     dh,24           ; end row 24
        test    ega,egaflag     ; 43 line wanted?
        jz      screen          ; no, set col
        mov     dh,49           ; end row 49 (43 for ega, 50 for vga)
screen:
        mov     dl,79           ; end col 80
        int     10h             ; BIOS call
        jmp     short endup
vga28:
;       test    ega,vgaflag     ; 28 line mode wanted?
;       jz      endup
        mov     ax,1202h        ; setup 400 scan lines
        mov     bl,30h
        int     10h
        mov     ax,0003h        ; setup normal text mode
        int     10h
        mov     ax,1111h        ; load ega character set
        mov     bl,00h
        int     10h
endup:
        .EXIT 0
        END
